/****************************************************************************
 ** This file is part of the MAIA (Malagasy Artificial Intelligence Art) Open Project.
 **
 ** Copyright (C) 2012, 2013 ANDRIAMILAMINA Monge.
 ** All rights reserved.
 **
 ** License: GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007.
 ** Author:  ANDRIAMILAMINA Monge.
 ** Contact: mongemilamina@gmail.com / +261 34 45 062 85.
 ** Year:    2013.
 ** Status:  MAIA Senior Engineer, Madagasikara, Antananarivo.
 ** Domain:  Automatic.
 **
 ** Commercial Usage
 ** Please contact MAIA.
 **
 ** GNU General Public License Usage
 ** This file may be used under the terms of the GNU
 ** General Public License version 3.0 as published by the Free Software
 ** Foundation and appearing in the file LICENSE.GPL included in the
 ** packaging of this file.  Please review the following information to
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** If you have questions regarding the use of this file, please contact
 ** the Author.
 **
 ****************************************************************************/

#include "unitconv.h"
#include "ui_unitconv.h"
#include <QMessageBox>
#include <QString>
#include <QIcon>
#include <QEvent>
#include <QCloseEvent>

UnitConv::UnitConv(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::UnitConv)
{
    ui->setupUi(this);

    myValidator = new QDoubleValidator();
    //#############################################################Lines
    ui->lineToConv->setText(tr("Value you want to convert..."));
    ui->lineToConv->selectAll();
    ui->lineConverted->setText(tr("Converted value..."));
    ui->lineToConv->setValidator(myValidator);
    //#############################################################Menus
    myActionNew = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_New.png"), tr("&New"), this);
    myActionNew->setShortcut(Qt::CTRL+Qt::Key_N);

    myActionCopyToConvertLine = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_Copy.png"), tr("&Copy") + " [ " + ui->lineToConv->text() + " ]", this);
    myActionCopyToConvertLine->setShortcut(Qt::CTRL+Qt::Key_C);

    myActionCopyConvertedLine = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_Copy.png"), tr("C&opy") + " [ " + ui->lineConverted->text() + " ]", this);
    myActionCopyConvertedLine->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_C);

    myActionAboutAndHelp = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_About.png"), tr("&About"), this);
    myActionAboutAndHelp->setShortcut(Qt::Key_F1);

    myActionQuit = new QAction(QIcon(":/qrc_icons/icons/MAIA_icon_Quit.png"), tr("&Quit"), this);
    myActionQuit->setShortcut(Qt::CTRL+Qt::Key_Q);
    //#############################################################Menus
    myMenuOperations = new QMenu(tr("Operations"));
    myMenuOperations->addAction(myActionNew);
    myMenuOperations->addSeparator();
    myMenuOperations->addAction(myActionQuit);

    myMenuCopies = new QMenu(tr("Copies"));
    myMenuCopies->addAction(myActionCopyToConvertLine);
    myMenuCopies->addAction(myActionCopyConvertedLine);

    myMenuAboutAndHelp = new QMenu("?");
    myMenuAboutAndHelp->addAction(myActionAboutAndHelp);

    ui->menuBar->addMenu(myMenuOperations);
    ui->menuBar->addMenu(myMenuCopies);
    ui->menuBar->addMenu(myMenuAboutAndHelp);

    //#############################################################
    connect(myActionNew, SIGNAL(triggered()), this, SLOT(new_()));
    connect(myActionAboutAndHelp, SIGNAL(triggered()), this, SLOT(aboutAndHelp()));
    connect(myActionQuit, SIGNAL(triggered()), this, SLOT(close()));

    connect(ui->lineToConv, SIGNAL(textChanged(QString)), this, SLOT(handleCopies()));
    connect(ui->lineConverted, SIGNAL(textChanged(QString)), this, SLOT(handleCopies()));

    connect(myActionCopyToConvertLine, SIGNAL(triggered()), this, SLOT(copyToConvertLine()));
    connect(myActionCopyConvertedLine, SIGNAL(triggered()), this, SLOT(copyConvertedLine()));
    //#############################################################
    connect(ui->b_degreCToDegreK, SIGNAL(clicked()), this, SLOT(celsiusToKelvin()));
    connect(ui->b_degreKToDegreC, SIGNAL(clicked()), this, SLOT(kelvinToCelsius()));
    //#############################################################
    connect(ui->b_metreToFoot, SIGNAL(clicked()), this, SLOT(meterToFoot()));
    connect(ui->b_footToMetre, SIGNAL(clicked()), this, SLOT(footToMeter()));
    //#############################################################
    //#############################################################
    connect(ui->b_nauticalToKilometre, SIGNAL(clicked()), this, SLOT(nauticalToKilMeter()));
    connect(ui->b_kilometreToNautical, SIGNAL(clicked()), this, SLOT(kiloMeterToNautical()));
    //#############################################################
    //#############################################################
    connect(ui->b_mileToKilometre, SIGNAL(clicked()), this, SLOT(mileToKiloMeter()));
    connect(ui->b_kilometreToMile, SIGNAL(clicked()), this, SLOT(kiloMeterToMile()));
    //#############################################################
    //#############################################################
    connect(ui->b_noeudToMetreparSeconde, SIGNAL(clicked()), this, SLOT(noeudsToMeterPerSec()));
    connect(ui->b_metreParSecondeToNoeud, SIGNAL(clicked()), this, SLOT(meterPerSecToNoeuds()));
    //#############################################################
    //#############################################################
    connect(ui->b_pouceTomillimetre, SIGNAL(clicked()), this, SLOT(inchToMilliMeter()));
    connect(ui->b_millimetreToPouce, SIGNAL(clicked()), this, SLOT(milliMeterToInch()));
    //#############################################################
    this->new_();
}
//#############################################################
void UnitConv::new_()
{
    ui->lineToConv->setText(tr("Value you want to convert..."));
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
    ui->lineConverted->setText(tr("Converted value..."));
    ui->lab_log->clear();

    this->handleCopies();//reset text of copies menu
}
//#############################################################
void UnitConv::handleCopies()
{
    myActionCopyToConvertLine->setText(tr("&Copy") + " [ " + ui->lineToConv->text() + " ]");
    myActionCopyConvertedLine->setText(tr("C&opy") + " [ " + ui->lineConverted->text() + " ]");
}
//#############################################################
void UnitConv::copyToConvertLine()
{
    ui->lineToConv->selectAll();
    ui->lineToConv->copy();
}
//#############################################################
void UnitConv::copyConvertedLine()
{
    ui->lineConverted->selectAll();
    ui->lineConverted->copy();
}
//#############################################################
void UnitConv::aboutAndHelp()
{
    myAboutLab = new QLabel(this);//create a label object for about dialog
    myAboutLab->setWordWrap(true);
    myAboutLab->setMargin(10);
    myAboutLab->setAlignment(Qt::AlignTop);
    myAboutLab->setText("\tMAIA Unit Conv 1.3 is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. \n\n\tMAIA Unit Conv is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details. \n\n\tYou should have received a copy of the GNU General Public License along with MAIA Unit Conv; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA");

    myHelpLab = new QLabel(this);//create a label object for help dialog
    myHelpLab->setWordWrap(true);
    myHelpLab->setMargin(10);
    myHelpLab->setAlignment(Qt::AlignTop);
    myHelpLab->setOpenExternalLinks(true);
    myHelpLab->setText("<html><head/><body> MAIA Unit Conv 1.3 helps you to convert a value to another specific one. <br> You just have to put the value you want to convert in the first line in the program and click on the appropriate button according to the response you expected. <br> If you want to copy the value you got, no more --Select-Copy-Paste-- stuffs, just go to the menu and copy what you want. <br><br><br> Author: <b>ANDRIAMILAMINA Monge</b>.<br>E-mail: <a href='mailto:mongemilamina@gmail.com'> mongemilamina@gmail.com</a>. <br> Web: <a href='https://www.andriamilamina.skyrock.com'> MAIA Website</a>.</body></html>");

    myTabWidget = new QTabWidget();//no parent, so this is gonna have its own window
    myTabWidget->setWindowFlags(Qt::WindowStaysOnTopHint);//make the tabwidget above other application windows
    myTabWidget->setAttribute(Qt::WA_ShowModal, true);//make the TabWidget Modal
    myTabWidget->insertTab(1, myAboutLab, QIcon(":/qrc_icons/icons/MAIA_icon_About.png"), tr("&About"));//adding the appropriate label to the tabBar, with its title
    myTabWidget->insertTab(1, myHelpLab, QIcon(":/qrc_icons/icons/MAIA_icon_Help.png"), tr("&Help"));//adding the appropriate label to the tabBar, with its title

    myTabWidget->setWindowTitle(tr("MAIA Unit Conv 1.3, About and Help"));
    myTabWidget->resize(510, 320);
    myTabWidget->setMinimumSize(510, 320);
    myTabWidget->show();
}
//#############################################################
void UnitConv::closeEvent(QCloseEvent *ev)//wait for a QCloseEvent *ev signal sent by close button click
{
    if(ev->type() == QEvent::Close)
    {
        QMessageBox::StandardButton reponse;
        reponse = QMessageBox::warning(this, tr("MAIA Unit Conv 1.3"), tr("Quit the application?"), QMessageBox::No | QMessageBox::Yes);
        if (reponse == QMessageBox::No)
        {
            ev->ignore();
            if(this->isHidden())
                this->show();
        }
        else if (reponse == QMessageBox::Yes)
            this->close();
    }
}
//#############################################################
//CALCULULATIONS start
//**********************************************************
//FootToMeter
void UnitConv::footToMeter()
{
    double val_foot = ui->lineToConv->text().toDouble();
    double val_metre = val_foot*0.304;
    QString resultat = QString::number(val_metre);

    ui->lineConverted->setText(resultat+ " " +tr("Meter(s)"));
    ui->lab_log->setText(ui->b_footToMetre->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//MeterToFoot
void UnitConv::meterToFoot()
{
    double val_metre = ui->lineToConv->text().toDouble();
    double val_foot = val_metre/0.304;
    QString resultat = QString::number(val_foot);

    ui->lineConverted->setText(resultat+ " " +tr("Feet"));
    ui->lab_log->setText(ui->b_metreToFoot->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//**********************************************************
//NauticalToKiloMeter
void UnitConv::nauticalToKilMeter()
{
    double val_nm = ui->lineToConv->text().toDouble();
    double val_km = val_nm*1.852;
    QString resultat = QString::number(val_km);

    ui->lineConverted->setText(resultat+ " " +tr("KiloMeter(s)"));
    ui->lab_log->setText(ui->b_nauticalToKilometre->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//KiloMeterToNautical
void UnitConv::kiloMeterToNautical()
{
    double val_km = ui->lineToConv->text().toDouble();
    double val_nm = val_km/1.852;
    QString resultat = QString::number(val_nm);

    ui->lineConverted->setText(resultat+ " " +tr("Nautical Mile(s)"));
    ui->lab_log->setText(ui->b_kilometreToNautical->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//**********************************************************
//InchToMilliMeter
void UnitConv::inchToMilliMeter()
{
    double val_inch = ui->lineToConv->text().toDouble();
    double val_mm = val_inch*25.4;
    QString resultat = QString::number(val_mm);

    ui->lineConverted->setText(resultat+ " " +tr("Millimeter(s)"));
    ui->lab_log->setText(ui->b_pouceTomillimetre->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//MilliMeterToInch
void UnitConv::milliMeterToInch()
{
    double val_mm = ui->lineToConv->text().toDouble();
    double val_inch = val_mm/25.4;
    QString resultat = QString::number(val_inch);

    ui->lineConverted->setText(resultat+ " " +tr("Inch(es)"));
    ui->lab_log->setText(ui->b_millimetreToPouce->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//**********************************************************
//CelsiusToKelvin
void UnitConv::celsiusToKelvin()
{
    double temp_deg_c = ui->lineToConv->text().toDouble(); //take the LineEdit content and convert it to double for calcul
    double temp_deg_k = temp_deg_c*273; //calcul
    QString resultat = QString::number(temp_deg_k); //convert the result  from double to a QString

    ui->lineConverted->setText(resultat+ " " +tr("Degree Kelvin")); //append the result on the result's lineEdit
    ui->lab_log->setText(ui->b_degreCToDegreK->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//KelvinToCelsius
void UnitConv::kelvinToCelsius()
{
    double temp_deg_k = ui->lineToConv->text().toDouble(); //take the LineEdit content and convert it to double for calcul
    double temp_deg_c = temp_deg_k/273; //calcul
    QString resultat = QString::number(temp_deg_c); //convert the result  from double to a QString

    ui->lineConverted->setText(resultat+ " " +tr("Degree Celsius")); //append the result on the result's lineEdit
    ui->lab_log->setText(ui->b_degreKToDegreC->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//**********************************************************
//MileToKiloMeter
void UnitConv::mileToKiloMeter()
{
    double val_mile = ui->lineToConv->text().toDouble();
    double val_km = val_mile*1.609;
    QString resultat = QString::number(val_km);

    ui->lineConverted->setText(resultat+ " " +tr("KiloMeter(s)"));
    ui->lab_log->setText(ui->b_mileToKilometre->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//KiloMeterToMile
void UnitConv::kiloMeterToMile()
{
    double val_km = ui->lineToConv->text().toDouble();
    double val_mile = val_km/1.609;
    QString resultat = QString::number(val_mile);

    ui->lineConverted->setText(resultat+ " " +tr("Mile(s)"));
    ui->lab_log->setText(ui->b_kilometreToMile->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//**********************************************************
//NoeudsToMeterPerSec
void UnitConv::noeudsToMeterPerSec()
{
    double val_kts = ui->lineToConv->text().toDouble();
    double val_mps = val_kts*0.514;
    QString resultat = QString::number(val_mps);

    ui->lineConverted->setText(resultat+ " " +tr("Meter(s) Per Seconds"));
    ui->lab_log->setText(ui->b_noeudToMetreparSeconde->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//MeterPerSecToNoeuds
void UnitConv::meterPerSecToNoeuds()
{
    double val_mps = ui->lineToConv->text().toDouble();
    double val_kts = val_mps/0.514;
    QString resultat = QString::number(val_kts);

    ui->lineConverted->setText(resultat+ " " +tr("KTS"));
    ui->lab_log->setText(ui->b_metreParSecondeToNoeud->text());
    ui->lineToConv->selectAll();
    ui->lineToConv->setFocus();
}
//**********************************************************
//#############################################################
UnitConv::~UnitConv()
{
    delete ui;
}
//#############################################################
void UnitConv::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}
